import imghdr
import io
import subprocess
import os.path
import tweepy
from tweebot.console import Console
from tweebot.keys import TwitterKeys


TWITTER_FILESIZE_LIMIT = 2999000 # About 3 Meg, we round down


TWITTER_FILESIZE_LIMIT = 2999000 # About 3 Meg, we round down
NEW_FILESIZE_LIMIT = '1999kb'
OUT_FILENAME = 'out.png'
JPG_FILENAME = 'out{}.jpg'


class TwitterClient(object):
    def __init__(self, keys, console=None):
        self.__keys = TwitterKeys.of(keys)
        self.__console = Console.of(console)
        self.__api = None

    @property
    def api(self):
        if self.__api is None:
            auth = tweepy.OAuthHandler(self.__keys.CONSUMER_KEY, self.__keys.CONSUMER_SECRET)
            auth.set_access_token(self.__keys.ACCESS_KEY, self.__keys.ACCESS_SECRET)
            self.__api = tweepy.API(auth)
        return self.__api

    def autofollow(self, follow: bool = True, unfollow: bool = True):
        if not (follow or unfollow):
            return
        followers = set(tweepy.Cursor(self.twitter.followers_ids).items())
        friends = set(tweepy.Cursor(self.twitter.friends_ids).items())
        print(followers)
        print(friends)

        if follow:
            followed = followers - friends
            print('Need to follow: {}'.format(followed))
        else:
            followed = set()

        if unfollow:
            unfollowed = friends - followers
            print('Need to unfollow: {}'.format(unfollowed))
        else:
            unfollowed = set()

        for user_id in followed:
            self.twitter.create_friendship(user_id)

        for user_id in unfollowed:
            self.twitter.destroy_friendship(user_id)

        return followed, unfollowed

    def tweet(self,
              status: str = None,
              filename: str = None,
              image: bytes = None
      ):
        """Tweet a status update with, optionally, an image.

        :param status: Status string to tweet.
        :param filename: Filename of an image to tweet.
        :param bytes: Bytes of an image to tweet, may use instead of filename.
        :return: None
        """
        if status is None and self.default_status is not None:
            status = self.default_status() if callable(self.default_status) else self.default_status

        if not self.args or not self.args.keys:
            if self.verbose > 0:
                print('No twitter keys registered, pipeline stopping')
            return

        if status:
            kwargs = dict(status=status)
        else:
            kwargs = dict()

        if image:
            if not filename:
                filename = 'tweet.{}'.format(imghdr.what(None, h=image))
            kwargs.update(file=io.BytesIO(image))
            with self.__console.timed(
                    'Updating twitter status ({}kb)...'.format(os.path.getsize(filename) // 1024),
                    'Updated status in {0:.3f}s'
            ):
                self.twitter.update_with_media(filename, **kwargs)
        elif filename:
            filename = self._resize(filename)

            with self.__console.timed(
                'Updating twitter status ({}kb)...'.format(os.path.getsize(filename) // 1024),
                'Updated status in {0:.3f}s'
            ):
                self.twitter.update_with_media(filename, **kwargs)
        elif status:
            self.twitter.update_status(**kwargs)
        else:
            raise RuntimeError('Tweet requires status or filename')

    def _convert(self, filename, outname=OUT_FILENAME):
        if filename == outname:
            return outname

        if not self.args or not self.args.magick:
            raise RuntimeError('No ImageMagick handler registered & it\'s required to convert! Pipeline stopping')

        with self.__console.timed('Converting image...', 'Converted image in {0:.3f}s'):
            call_args = [
                self.args.magick, filename, outname
            ]
            subprocess.check_call(call_args)

        return outname

    def _resize(self, filename):
        if not self.args or not self.args.magick:
            raise RuntimeError('No ImageMagick handler registered & it\'s required to resize! Pipeline stopping')

        file_size = os.path.getsize(filename)

        attempt = 0
        while file_size > TWITTER_FILESIZE_LIMIT and attempt < 5:
            with self.__console.timed('Needs more jpeg...', 'Resized image in {0:.3f}s'):
                new_filename = JPG_FILENAME.format(attempt)
                call_args = [
                    self.args.magick, filename,
                    '-define', 'jpeg:extent={}'.format(NEW_FILESIZE_LIMIT),
                ]
                if attempt > 0:
                    call_args.extend(['-scale', '70%'])
                call_args.append(new_filename)
                subprocess.check_call(call_args)
                filename = new_filename
                file_size = os.path.getsize(filename)

        return filename



from collections import namedtuple
import argparse
import sys
from tweebot.client import TwitterClient
from tweebot.console import Console


Directives = namedtuple('Directives', ['filename', 'directive'])


DEFAULT_DIRECTIVE = ']'


import pkg_resources
try:
    VERSION = pkg_resources.require("tweebot")[0].version
except:
    VERSION = 'DEV'


def make_parser(parser=None, subparsers=None):
    if parser is None:
        parser = argparse.ArgumentParser(
            description='Tweebot: A simple twitter bot\n  Version {}'.format(VERSION),
            formatter_class=argparse.RawDescriptionHelpFormatter
        )
    if subparsers is None:
        subparsers = parser.add_subparsers()

    parser.add_argument(
        '--keys', '-k', type=str,
        help='Keys file needed to post to Twitter',
    )
    parser.add_argument(
        '--verbose', '-v', action='count',
        help='More v\'s, more verbose'
    )
    parser.add_argument(
        '--version', action='store_true',
        help='Print version ({}) and exit'.format(VERSION)
    )

    tweet_parser = subparsers.add_parser('tweet', help='Tweet a status update')
    _make_pipeline_parser(tweet_parser, 'tweet')
    tweet_parser.set_defaults(handler=do_tweet)

    print_parser = subparsers.add_parser('print', help='Print a status update without sending -- can be piped')
    _make_pipeline_parser(print_parser, 'print')
    print_parser.set_defaults(handler=do_print)

    follow_parser = subparsers.add_parser('follow', help='Follow users, or auto follow your followers')
    follow_parser.add_argument(
        'user_ids', type=str, nargs='*',
        help='User ids to follow'
    )
    follow_parser.add_argument(
        '--auto', '-A', action='store_true',
        help='Auto-follow your followers, unfollow unfollowers'
    )
    follow_parser.set_defaults(handler=do_follow)

    return parser


def _make_pipeline_parser(parser, verb):
    parser.add_argument(
        'status', type=str, nargs='*',
        help='Text status to {verb}: "enclose multiple words in quotes", use - to pipe input'
        .format(verb=verb)
    )
    parser.add_argument(
        '--filename', '-f', type=str, default=None,
        help='Path to image/media file to {verb}, if any'.format(verb=verb)
    )
    parser.add_argument(
        '--directive', '-D', type=str, default=DEFAULT_DIRECTIVE,
        help=('Escape sequence that, when appearing at the beginning of a status line, modifies {verb} arguments,' +
              ' allowing for example setting the media filename from within the status text').format(verb=verb)
    )


def loads_status(status, directives):
    """
    Returns status and dict of directives from status string and directive info.

    :param args:  Parsed args
    :return: tuple of (status string, directive dict)
    """
    if status == '-':
        status = sys.stdin.read()

    if directives.directive:
        status_lines = list()
        for line in status.splitlines(True):
            stripped = line.strip()
            if directives.directive and stripped.startswith(directives.directive) and value:
                arg, value = stripped[len(directives.directive):].split(None, 1)
                directives = directives._replace(**{arg: value})
            else:
                status_lines.append(line)
        status = ''.join(status_lines)

    return status, directives


def dumps_status(status, directives):
    initial_directive = DEFAULT_DIRECTIVE
    directive = initial_directive
    for line in status.splitlines(True):
        while line.startswith(directive):
            directive += DEFAULT_DIRECTIVE

    status_lines = list()
    if initial_directive != directive:
        status_lines.append('{}directive {}\n'.format(initial_directive, directive))

    for key, value in sorted(directives._asdict().items()):
        if key != 'directive' and value:
            status_lines.append('{}{} {}\n'.format(directive, key, value))

    status_lines.append(status)

    return ''.join(status_lines)


def generate_status(args, generators=()):
    status, directives = loads_status(' '.join(args.status), Directives(args.filename, args.directive))
    for generator in generators:
        generated = generator(status, directives)
        if isinstance(generated, dict):
            status = generated.pop('status')
            directives = generated
        elif isinstance(generated, tuple):
            status, directives = generated
        else:
            status, directives = loads_status(generated, directives)

    return status, directives


def do_tweet(args, generators=()):
    status, directives = generate_status(args, generators)

    client = TwitterClient(args.keys, console=Console(args.verbose))
    client.tweet(status, filename=directives.filename)


def do_print(args, generators=()):
    status, directives = generate_status(args, generators)
    print(dumps_status(status, directives))


def do_follow(args, generator=None):
    client = TwitterClient(args.keys, console=Console(args.verbose))
    if args.user_ids:
        raise NotImplementedError('Haven\'t implemented individual follows yet')
    if args.auto:
        client.autofollow()


def main(
        generators=()
):
    parser = make_parser()
    args = parser.parse_args()
    if args.version:
        print(VERSION)
        return
    if callable(generators):
        generators = [generators]

    parser.handler(args, generators)

if __name__ == '__main__':
    main()



import os
from setuptools import setup, find_packages
from pip.req import parse_requirements
from pip.download import PipSession

version = '0.2.1'

def read(f):
    return open(os.path.join(os.path.dirname(__file__), f)).read().strip()

try:
   import pypandoc
   description = pypandoc.convert('README.md', 'rst')
except (IOError, ImportError):
   description = read('README.md')

setup(name='tweebot',
      version=version,
      description='A simple twitter-bot command-line tool and library',
      long_description=description,
      classifiers=[
          'License :: OSI Approved :: MIT License',
          'Intended Audience :: Other Audience',
          'Programming Language :: Python :: 3'],
      author='K.C.Saff',
      author_email='kc@saff.net',
      url='https://github.com/kcsaff/tweebot',
      license='MIT',
      packages=find_packages(),
      install_requires=[
          'tweepy>=3.5.0'
      ],
      entry_points={
          'console_scripts': ['tweebot = tweebot:main']
      },
      include_package_data=False,
)
